@@ -82,12 +82,19 @@ static char sccsid[] = "@(#)commands.c	5.5 (Berkeley) 3/22/91";
 
 #ifdef USE_SSL
 
+SSL_CTX *ssl_ctx;
 SSL *ssl_con;
 int ssl_debug_flag=0;
 int ssl_only_flag=0;
 int ssl_active_flag=0;
 int ssl_verify_flag=SSL_VERIFY_NONE;
 int ssl_secure_flag=0;
+int ssl_certsok_flag=0;
+char *cert_file=NULL;
+char *key_file=NULL;
+char *cipher_list=NULL;
+
+extern int display_connect_details();
 
 #endif /* USE_SSL */
 
@@ -1667,7 +1674,7 @@ env_find(var)
 	return(NULL);
 }
 
-	void
+void
 env_init()
 {
 	extern char **environ;
@@ -1714,7 +1721,7 @@ env_init()
 	env_export((unsigned char *)"PRINTER");
 }
 
-	struct env_lst *
+static	struct env_lst *
 env_define(var, value)
 	unsigned char *var, *value;
 {
@@ -1739,7 +1746,7 @@ env_define(var, value)
 	return(ep);
 }
 
-	void
+static	void
 env_undefine(var)
 	unsigned char *var;
 {
@@ -1757,7 +1764,7 @@ env_undefine(var)
 	}
 }
 
-	void
+static	void
 env_export(var)
 	unsigned char *var;
 {
@@ -1767,7 +1774,7 @@ env_export(var)
 		ep->export = 1;
 }
 
-	void
+static	void
 env_unexport(var)
 	unsigned char *var;
 {
@@ -1777,7 +1784,7 @@ env_unexport(var)
 		ep->export = 0;
 }
 
-	void
+static	void
 env_send(var)
 	unsigned char *var;
 {
@@ -1800,7 +1807,7 @@ env_send(var)
 	env_opt_end(0);
 }
 
-	void
+static	void
 env_list()
 {
 	register struct env_lst *ep;
@@ -1854,8 +1861,8 @@ struct authlist {
 };
 
 extern int
-	auth_enable P((int)),
-	auth_disable P((int)),
+	auth_enable P((char *)),
+	auth_disable P((char *)),
 	auth_status P((void));
 static	auth_help P((void));
 
@@ -1893,6 +1900,12 @@ auth_cmd(argc, argv)
 {
     struct authlist *c;
 
+    if (argc < 2) {
+      fprintf(stderr,
+          "Need an argument to 'auth' command.  'auth ?' for help.\n");
+      return 0;
+    }
+
     c = (struct authlist *)
 		genget(argv[1], (char **) AuthList, sizeof(struct authlist));
     if (c == 0) {
@@ -1948,7 +1961,7 @@ struct encryptlist EncryptList[] = {
 						EncryptEnable, 1, 1, 2 },
     { "disable", "Disable encryption. ('encrypt enable ?' for more)",
 						EncryptDisable, 0, 1, 2 },
-    { "type", "Set encryptiong type. ('encrypt type ?' for more)",
+    { "type", "Set encryption type. ('encrypt type ?' for more)",
 						EncryptType, 0, 1, 1 },
     { "start", "Start encryption. ('encrypt start ?' for more)",
 						EncryptStart, 1, 0, 1 },
@@ -1992,6 +2005,12 @@ encrypt_cmd(argc, argv)
 {
     struct encryptlist *c;
 
+    if (argc < 2) {
+       fprintf(stderr,
+           "Need an argument to 'encrypt' command.  'encrypt ?' for help.\n");
+       return 0;
+    }
+
     c = (struct encryptlist *)
 		genget(argv[1], (char **) EncryptList, sizeof(struct encryptlist));
     if (c == 0) {
@@ -2319,8 +2338,11 @@ tn(argc, argv)
 		tos = 020;	/* Low Delay bit */
 	    if (tos
 		&& (setsockopt(net, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int)) < 0)
-		&& (errno != ENOPROTOOPT))
-		    perror("telnet: setsockopt (IP_TOS) (ignored)");
+		&& (errno != ENOPROTOOPT)) {
+		    /* DGUX restricts this operation to root only */
+		    if (errno!=EACCES)
+			perror("telnet: setsockopt (IP_TOS) (ignored)");
+	    }
 	}
 #endif	/* defined(IPPROTO_IP) && defined(IP_TOS) */
 
@@ -2369,27 +2391,51 @@ tn(argc, argv)
 	    SSL_LOG=SSL_ERR=NULL;
 	}
 
-	ssl_con=(SSL *)SSL_new();
+	/* init things so we will get meaningful error messages
+	 * rather than numbers 
+	 */
+	SSL_load_error_strings();
+
+	ssl_ctx=(SSL_CTX *)SSL_CTX_new();
+	ssl_con=(SSL *)SSL_new(ssl_ctx);
 
         SSL_set_fd(ssl_con,net);
         SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
-	SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
+	if (cipher_list==NULL)
+	    SSL_set_pref_cipher(ssl_con,getenv("SSL_CIPHER"));
+	else
+	    SSL_set_pref_cipher(ssl_con,cipher_list);
 
 	/* Add in any certificates if you want to here ... */
+	if (cert_file) {
+	    if (!SSL_use_certificate_file(ssl_con, cert_file,
+		X509_FILETYPE_PEM)) {
+		fprintf(stderr,"%s: ",cert_file);
+		ERR_print_errors(stderr);
+		exit(1);
+	    } else {
+		if (!key_file)
+		    key_file = cert_file;
+		if (!SSL_use_RSAPrivateKey_file(ssl_con, key_file,
+		    X509_FILETYPE_PEM)) {
+		    fprintf(stderr,"%s: ", key_file);
+		    ERR_print_errors(stderr);
+		    exit(1);
+		}
+	    }
+	}
 
 	if (ssl_only_flag) {
 	    if (!SSL_connect(ssl_con)) {
                 static char errbuf[1024];
 
-	        sprintf(errbuf,"telnet: SSL_connect error %s %s\n",
-		            SSL_error_func_string(SSL_errno),
-			    SSL_error_string(SSL_errno));
+		ERR_print_errors(stderr);
+		perror("SSL_connect");
+		fflush(stderr);
 
-		perror(errbuf);
 		return 0;
 	    } else {
-		fprintf(stderr,"[SSL Cipher %s]\n",SSL_get_cipher(ssl_con));
-		fflush(stderr);
+		display_connect_details(ssl_con,1);
 		ssl_active_flag=1;
 	    }
 	}
