@@ -176,7 +176,8 @@ main(argc, argv)
 			    fflush(stderr);
 #endif /* USE_SOCKS */
 			}
-			if (strcmp(optarg, "!socks") == 0 ) {
+			if ( (strcmp(optarg, "!socks") == 0 ) ||
+			     (strcmp(optarg, "nosocks") == 0 ) ) {
 #ifdef USE_SOCKS
 			    socks_flag=0;
 #else /* !USE_SOCKS */
@@ -198,7 +199,8 @@ main(argc, argv)
 			    fflush(stderr);
 #endif /* USE_SOCKS */
 			}
-			if (strcmp(optarg, "!socks") == 0 ) {
+			if ( (strcmp(optarg, "!socks") == 0 ) ||
+			     (strcmp(optarg, "nosocks") == 0 ) ) {
 #ifdef USE_SOCKS
 			    socks_flag=0;
 #else /* !USE_SOCKS */
