@@ -69,6 +69,9 @@ int	auth_level = 0;
 int	require_SecurID = 0;
 #endif
 
+#include <sys/ioctl.h>
+
+
 /*
  * I/O data buffers,
  * pointers, and counters.
@@ -91,18 +94,23 @@ extern void usage P((void));
 
 #ifdef USE_SSL
 
+/*
 #include "ssl_err.h"
+*/
 
 SSL *ssl_con;
+SSL_CTX *ssl_ctx;
 int ssl_debug_flag=0;
 int ssl_only_flag=0;
 int ssl_active_flag=0;
 int ssl_secure_flag=0;
+int ssl_certsok_flag=0;
 int ssl_verify_flag=SSL_VERIFY_NONE;
 char ssl_file_path[1024];    /* don't look at that nasty value to the left */
 
 static char *my_ssl_key_file=NULL;
 static char *my_ssl_cert_file=NULL;
+char *cipher_list;
 
 #endif /* USE_SSL */
 
@@ -150,6 +158,9 @@ main(argc, argv)
 		        if (strcmp(optarg, "ssl") == 0 ) {
 			    ssl_only_flag=1;
 			}
+		        if (strcmp(optarg, "certsok") == 0 ) {
+			    ssl_certsok_flag=1;
+			}
 		        if (strcmp(optarg, "secure") == 0 ) {
 			    ssl_secure_flag=1;
 			}
@@ -163,6 +174,10 @@ main(argc, argv)
 		        if (strncmp(optarg, "key=", strlen("key=")) == 0 ) {
 			    my_ssl_key_file=optarg+strlen("key=");
 			}
+
+		        if (strncmp(optarg,"cipher=",sizeof("cipher"))==0) {
+			    cipher_list=optarg+sizeof("cipher");
+			}
 			break;
 #endif /* USE_SSL */
 
@@ -351,10 +366,14 @@ main(argc, argv)
 		/* disable all the debug and trace */
 		SSL_LOG=SSL_ERR=NULL;
 	    }
+	    SSL_load_error_strings();
 
-            if (!X509_set_default_verify_paths()) {
+	    ssl_ctx=(SSL_CTX *)SSL_CTX_new();
+
+            if (!X509_set_default_verify_paths(ssl_ctx->cert)) {
 		fprintf(stderr,"telnetd: cannot load certificates via X509_set_default_verify_paths\n");
 		fflush(stderr);
+		ERR_print_errors(stderr);
 		exit(1);
 	    }
 
@@ -463,53 +482,54 @@ main(argc, argv)
 
 #ifdef USE_SSL
         /* do the SSL stuff now ... before we play with pty's */
-	ssl_con=(SSL *)SSL_new();
+	ssl_con=(SSL *)SSL_new(ssl_ctx);
 	SSL_set_fd(ssl_con,0);
 
-        /* just use telnetd for public and private and don't
+        /* just use telnetd.pem for public and private and don't
 	 * encrypt it with a pass phrase 
 	 */
-	sprintf(ssl_file_path,"%s/private/%s",X509_get_default_cert_area(),
-	        "telnetd.key");
+	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
+	        "telnetd.pem");
 	if (SSL_use_RSAPrivateKey_file(ssl_con,
 	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file,
 		X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_RSAPrivateKey_file error %s:%s\n",
-		    SSL_error_func_string(SSL_errno),
-		    SSL_error_string(SSL_errno));
-	    /* if there was a system error then display it */
-	    if (SSL_errno==SSL_R_ERRNO)
-		perror("SSL_use_RSAPrivateKey_file");
+	    fprintf(stderr,"SSL_use_RSAPrivateKey_file(%s) error ",
+	        my_ssl_key_file==NULL?ssl_file_path:my_ssl_key_file);
+	    ERR_print_errors(stderr);
 	    fflush(stderr);
 	    sleep(5);
 	    _exit(1);
 	}
 
 	sprintf(ssl_file_path,"%s/%s",X509_get_default_cert_dir(),
-	        "telnetd.cert");
+	        "telnetd.pem");
 	if (SSL_use_certificate_file(ssl_con,
 	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file,
 	        X509_FILETYPE_PEM)<=0) {
-	    fprintf(stderr,"SSL_use_certificate_file error %s:%s\n",
-		    SSL_error_func_string(SSL_errno),
-		    SSL_error_string(SSL_errno));
-	    /* if there was a system error then display it */
-	    if (SSL_errno==SSL_R_ERRNO)
-		perror("SSL_use_certificate_file");
+	    fprintf(stderr,"SSL_use_certificate_file(%s) error ",
+	        my_ssl_cert_file==NULL?ssl_file_path:my_ssl_cert_file);
+	    ERR_print_errors(stderr);
 	    fflush(stderr);
 	    sleep(5);
 	    _exit(1);
 	}
 
-	SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
+	/* if we gave a cipher list then ... */
+        if (cipher_list!=NULL)
+	    SSL_set_pref_cipher(ssl_con,cipher_list);
 
 	if (ssl_only_flag) {
-	    if (!SSL_accept(ssl_con)) {
+	    /* hmm ... only when running talking to things like
+	     * https servers should we hit this code and then
+	     * we really don't care *who* we talk to :-)
+	     */
+	    SSL_set_verify(ssl_con,ssl_verify_flag,NULL);
+
+	    if (SSL_accept(ssl_con) <= 0) {
 		static char errbuf[1024];
 	    
-	        sprintf(errbuf,"SSL_accept error %s:%s\n",
-		    SSL_error_func_string(SSL_errno),
-		    SSL_error_string(SSL_errno));
+	        sprintf(errbuf,"SSL_accept error %s\n",
+		    ERR_error_string(ERR_get_error(),NULL));
 
 		syslog(LOG_WARNING, errbuf);
 
@@ -607,7 +627,7 @@ getterminaltype(name)
     /* if SSL is required then we will stop if we don't
      * have it *now*
      */
-    if (ssl_secure_flag && !his_state_is_will(TELOPT_AUTHENTICATION)) {
+    if (ssl_secure_flag) {
 	if (!ssl_active_flag) {
 	    /* we need to indicate to the user that SSL
 	     * is required ... need to think about how
@@ -776,6 +796,19 @@ terminaltypeok(s)
     if (terminaltype == NULL)
 	return(1);
 
+/* under Linux if the terminal type is not known then we end up
+ * with a perror() to stderr occuring which stuffs things up
+ * if we are running SSL
+ *
+ * Thanks to:
+ *    bogk@inf.fu-berlin.de (Andreas Bogk) for finding this
+ *
+ *
+ */
+#ifdef LINUX
+    return (1);
+#endif /* LINUX */
+
     /*
      * tgetent() will return 1 if the type is known, and
      * 0 if it is not known.  If it returns -1, it couldn't
@@ -783,8 +816,7 @@ terminaltypeok(s)
      * it won't help to say we failed, because we won't be
      * able to verify anything else.  So, we treat -1 like 1.
      */
-    if (tgetent(buf, s) == 0)
-	return(0);
+    if (tgetent(buf, s) == 0) return(0);
     return(1);
 }
 
